#include "StdAfx.h"
#include<GL/gl.h>
#include<GL/glu.h>

#include "moebiusstrip.h"


MoebiusStrip::MoebiusStrip(void)
{
 SideNum=3;
 Turns=1;
 Radius=3;
 InnerRadius=1;
 FilletRatio=.1f;
 StartTwistDeg=0;
 StepNum=36;
}

MoebiusStrip::~MoebiusStrip(void)
{
}

void MoebiusStrip::Clear()
{
	Ring.Clear();
}

void MoebiusStrip::Generate()
{
	vector<Point3f> P;
	vector<Point3f> N;

	GenerateSection(P,N,SideNum,InnerRadius,FilletRatio );
	GenerateRing(P,N);
}
//
//
//
//
//
//
//




void MoebiusStrip::GenerateRing(const vector<Point3f> &SP, const vector<Point3f> &SN)
{
	const float BlockSide=3;
	float StepAngleDeg=360.0f/StepNum;
	float StepTwistDeg=float((360.0f/SideNum)*Turns)/StepNum;

	// Inizializzazione della Modelview Matrix
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();	
  	
	// -- Disegno dell'Anello -- 
  // il disegno viene fatto a sezioni, 

	for(int i=0;i<StepNum+1;++i)
	{
		glPushMatrix();
		glRotatef(i*StepAngleDeg    , 0,0,1);   // Rotazione Intorno all'asse principale
		glTranslatef(Radius,0,0);
	  glRotatef(i*StepTwistDeg, 0,1,0);   // Avvolgimento della striscia.
		Matrix44d mm;
		glGetModelViewMatrix(mm);
		glPopMatrix();
		for(unsigned int j=0;j<SP.size();++j)
		{
			Point4d p4=mm*Point4d(SP[j][0],SP[j][1],SP[j][2],1); 
			Point4d n4=mm*Point4d(SN[j][0],SN[j][1],SN[j][2],0);
			Ring.vert.push_back(CIVertex(
				Point3f(float(p4[0]),float(p4[1]),float(p4[2])),
				Point3f(float(n4[0]),float(n4[1]),float(n4[2])),float(i),float(j)));
		}
	}
  glPopMatrix();
  //  a--c
	//  |\ |
	//  | \|
	//  b--d  
	//
	unsigned int sz=SP.size();
	for(int i=0;i<StepNum;++i)
	{
		for(unsigned int k=0;k<sz;++k)
		{
			CIFace tmp;

			tmp.v[0]=&(Ring.vert[(i+0)*sz+(k+0)%sz]); // a 
			tmp.v[1]=&(Ring.vert[(i+1)*sz+(k+1)%sz]); // d 
			tmp.v[2]=&(Ring.vert[(i+0)*sz+(k+1)%sz]); // b 
			Ring.face.push_back(tmp);

			tmp.v[0]=&(Ring.vert[(i+0)*sz+(k+0)%sz]); // a
			tmp.v[1]=&(Ring.vert[(i+1)*sz+(k+0)%sz]); // c
			tmp.v[2]=&(Ring.vert[(i+1)*sz+(k+1)%sz]); // d
			Ring.face.push_back(tmp);
		}
	}
}


void MoebiusStrip::Move(MoebiusStrip::Pos &p, float DistDeg)
{
	p.ThetaDeg+=DistDeg;
	if(p.ThetaDeg >360) 
	{
		p.ThetaDeg -= 360.0;
		p.CurSide += int(Turns);
	}
	if(p.ThetaDeg <0) 
	{
		p.ThetaDeg += 360.0;
		p.CurSide -= int(Turns);
	}
}
void MoebiusStrip::GenerateSection( vector<Point3f> &SP,  vector<Point3f> &SN, int side ,float r, float fillet)
{
 assert(side>1);
 if(fillet<0) fillet=0;
 if(fillet>1) fillet=1;
 
 float TurnStepDeg=360.0f/side;
 float StripAngleDeg=TurnStepDeg*(1.0f-fillet);
 float HalfFilletAngleDeg=(TurnStepDeg*fillet)/2;
 SP.clear();SN.clear();
 float BaseAngleDeg = StartTwistDeg-TurnStepDeg/2.0f;
 for(int i=0;i<side;++i)
 {
  float a1Rad=ToRad(BaseAngleDeg+i*TurnStepDeg + HalfFilletAngleDeg);
	float a2Rad=ToRad(BaseAngleDeg+(i+1)*TurnStepDeg - HalfFilletAngleDeg);
  float a3Rad=ToRad(BaseAngleDeg+(i+1)*TurnStepDeg + HalfFilletAngleDeg);
  float n1Rad=ToRad(BaseAngleDeg+i*TurnStepDeg + TurnStepDeg/2.0f);
	float n2Rad=ToRad(BaseAngleDeg+(i+1)*TurnStepDeg);
	SP.push_back(Point3f(Cos(a1Rad),0,Sin(a1Rad))*r); SN.push_back(Point3f(Cos(n1Rad),0,Sin(n1Rad)));
	SP.push_back(Point3f(Cos(a2Rad),0,Sin(a2Rad))*r); SN.push_back(Point3f(Cos(n1Rad),0,Sin(n1Rad)));
	SP.push_back(Point3f(Cos(a2Rad),0,Sin(a2Rad))*r); SN.push_back(Point3f(Cos(n2Rad),0,Sin(n2Rad)));
	SP.push_back(Point3f(Cos(a3Rad),0,Sin(a3Rad))*r); SN.push_back(Point3f(Cos(n2Rad),0,Sin(n2Rad)));
 }
	

}

void MoebiusStrip::Transform(const MoebiusStrip::Pos &p)
{
	// di quanto si gira per ogni turn (e.g. 
	float TurnStepDeg = (360.0f/SideNum);
	// di quanto si avvolge per ogni degree
	float TwistPerDeg = (TurnStepDeg * Turns )/360.0f;
	float HalfFilletAngleDeg=(TurnStepDeg*FilletRatio)/2;
 	float BaseAngleDeg = StartTwistDeg-TurnStepDeg/2.0f;
	glRotatef(p.ThetaDeg, 0,0,1);   // Rotazione Intorno all'asse principale
	glTranslatef(Radius,0,0);
	glRotatef(p.ThetaDeg*TwistPerDeg, 0,1,0);   // Avvolgimento della striscia.
	glTranslatef(Cos(ToRad(BaseAngleDeg+HalfFilletAngleDeg))*InnerRadius,0,0);
}

